<?php

namespace Drupal\workbench_approver\Access;

use Drupal\content_moderation\Access\LatestRevisionCheck;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\RevisionableInterface;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\user\EntityOwnerInterface;
use Drupal\workbench_approver\Entity\ApproverInterface;
use Symfony\Component\Routing\Route;

/**
 * Adds assigned reviewers access to the latest revision tab.
 */
class ApproveLatestRevisionCheck extends LatestRevisionCheck implements AccessInterface {

  /**
   * {@inheritdoc}
   */
  public function access(Route $route, RouteMatchInterface $route_match, AccountInterface $account) {
    // @todo Should we just call parent::access() here?
    // This tab should not show up unless there's a reason to show it.
    $entity = $this->loadEntity($route, $route_match);
    if ($this->moderationInfo->hasPendingRevision($entity) && $this->moderationInfo->isModeratedEntity($entity)) {
      // Check the global permissions first.
      $access_result = AccessResult::allowedIfHasPermissions($account, [
        'view latest version',
        'view any unpublished content',
      ]);
      if (!$access_result->isAllowed()) {
        // Check entity owner access.
        $owner_access = AccessResult::allowedIfHasPermissions($account, [
          'view latest version',
          'view own unpublished content',
        ]);
        $owner_access = $owner_access->andIf((AccessResult::allowedIf($entity instanceof EntityOwnerInterface && ($entity->getOwnerId() == $account->id()))));
        $access_result = $access_result->orIf($owner_access);
        // This is our new check; it expands the allowed list.
        if (!$access_result->isAllowed() && $this->checkAssignments($entity, $account)) {
          $access_result = AccessResult::allowedIfHasPermission($account, 'use workbench approver');
          // @todo Cache by $approver entity?
          $access_result->addCacheableDependency($account);
        }
      }

      return $access_result->addCacheableDependency($entity);
    }

    return AccessResult::forbidden('No pending revision for moderated entity.')->addCacheableDependency($entity);
  }

  /**
   * Checks if a user is assigned to approve the entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity being checked.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user making the request.
   *
   * @return bool
   *   TRUE if the user is assigned as a reviewer.
   */
  public function checkAssignments(EntityInterface $entity, AccountInterface $account): bool {
    $return = FALSE;

    // Ensure that we have an entity in moderation.
    assert($entity instanceof RevisionableInterface);
    $workflow = $this->moderationInfo->getWorkflowForEntity($entity);
    if ($workflow !== NULL) {
      // @todo Can we inject this service?
      // @phpstan-ignore-next-line
      $storage = \Drupal::entityTypeManager()->getStorage('approver');

      $values = [
        'content_entity_type_id' => $entity->getEntityTypeId(),
        'content_entity_id' => $entity->id(),
        'content_entity_revision_id' => $entity->getRevisionId(),
        'approver_uid' => $account->id(),
        'workflow_id' => $workflow->id(),
        'langcode' => $entity->language()->getId(),
      ];

      $assigned = $storage->loadByProperties($values);

      // Assignments (not approvals) are active if > 0.
      if (count($assigned) > 0) {
        $record = current($assigned);
        assert($record instanceof ApproverInterface);
        if ($record->getStatus() > ApproverInterface::APPROVAL_RESCINDED) {
          $return = TRUE;
        }
      }
    }

    return $return;
  }

}
